import addTypeControl from "./typeControl";
import addZoomControl from "./zoomControl";

// 初始化 百度自带的 定位、缩放控件
export function initControls({
  map,
  BMapGL,
  locationControl = false,
  zoomControl = false,
}) {
  if (locationControl) {
    var scaleCtrl = new BMapGL.LocationControl(
      typeof locationControl === "object"
        ? {
            anchor: locationControl.anchor,
            ...(Array.isArray(locationControl.offset) && {
              offset: new BMapGL.Size(...locationControl.offset),
            }),
          }
        : {}
    ); // 添加比例尺控件
    map.addControl(scaleCtrl);
  }
  if (zoomControl) {
    var zoomCtrl = new BMapGL.ZoomControl(
      typeof zoomControl === "object"
        ? {
            anchor: zoomControl.anchor,
            ...(Array.isArray(zoomControl.offset) && {
              offset: new BMapGL.Size(...zoomControl.offset),
            }),
          }
        : {}
    ); // 添加缩放控件
    map.addControl(zoomCtrl);
  }
}

// 初始化 自定义控件 地图类型、缩放控件
export function initCustomControl({
  map,
  BMapGL,
  mapOptions,
  custonZoom = 1,
  customZoomControl = true,
  customTypeControl = true,
}) {
  if (customZoomControl) {
    addZoomControl({
      map,
      BMapGL,
      anchor: customZoomControl.anchor || 3, // 处理稍微有点不好
      offset: customZoomControl.offset || [20, 20],
      zoom: custonZoom,
    });
  }
  if (customTypeControl) {
    addTypeControl({
      map,
      BMapGL,
      anchor: customTypeControl.anchor || 1,
      offset: customTypeControl.offset || [20, 20],
      mapOptions,
    });
  }
}

export default ({
  map,
  BMapGL,
  mapOptions,
  custonZoom = 1,
  locationControl = false,
  zoomControl = false,
  customZoomControl = true,
  customTypeControl = true,
}) => {
  initControls({ map, BMapGL, locationControl, zoomControl });
  initCustomControl({
    map,
    BMapGL,
    mapOptions,
    custonZoom,
    customZoomControl,
    customTypeControl,
  });
};
