import('./type.css')

// 添加自定义地图类型控件
export default ({ map, BMapGL, anchor, offset, mapOptions }) => {
  let mapTypeEarth = false
  if (mapOptions && mapOptions.mapType === 'B_EARTH_MAP') {
    mapTypeEarth = true
  }
  //定义一个控件类
  function MapTypeControl() {
    this.defaultAnchor = anchor;
    this.defaultOffset = new BMapGL.Size(...offset);
  }
  //通过JavaScript的prototype属性继承于BMap.Control
  MapTypeControl.prototype = new BMapGL.Control();

  //自定义控件必须实现自己的initialize方法，并且将控件的DOM元素返回
  //在本方法中创建个div元素作为控件的容器，并将其添加到地图容器中
  MapTypeControl.prototype.initialize = function (map) {
    //创建一个dom元素
    var div = document.createElement("div");
    const divSatellite = document.createElement("div");
    const divNormal = document.createElement("div");
    divSatellite.innerHTML = "卫星";
    divNormal.innerHTML = "地图";
    //添加文字说明
    div.appendChild(divSatellite);
    div.appendChild(divNormal);
    // 设置样式
    div.classList.add("custom-type-box");
    divSatellite.classList.add("custom-type-control");
    divNormal.classList.add("custom-type-control");
    divSatellite.style.background = "";
    divNormal.style.background = "#2b6fb5";
    divSatellite.style.color = "#000";
    divNormal.style.color = "#fff";
    if (mapTypeEarth) {
      divSatellite.style.background = "#2b6fb5";
      divNormal.style.background = "";
      divSatellite.style.color = "#fff";
      divNormal.style.color = "#000";
    } else {
      divSatellite.style.background = "";
      divNormal.style.background = "#2b6fb5";
      divSatellite.style.color = "#000";
      divNormal.style.color = "#fff";
    }
    // 绑定事件
    divSatellite.onclick = function (e) {
      divSatellite.style.background = "#2b6fb5";
      divNormal.style.background = "#fff";
      divSatellite.style.color = "#fff";
      divNormal.style.color = "#000";
      divSatellite.style.borderColor = "rgb(112 109 109)";
      divNormal.style.borderColor = "rgb(112 109 109)";
      map.setMapType(BMAP_SATELLITE_MAP);
    };
    divNormal.onclick = function (e) {
      divSatellite.style.background = "#fff";
      divNormal.style.background = "#2b6fb5";
      divSatellite.style.color = "#000";
      divNormal.style.color = "#fff";
      divSatellite.style.borderColor = "#c5c7cf";
      divNormal.style.borderColor = "#c5c7cf";
      map.setMapType(BMAP_NORMAL_MAP);
    };
    // 添加DOM元素到地图中
    map.getContainer().appendChild(div);
    // 将DOM元素返回
    return div;
  };

  //创建控件元素
  var myZoomCtrl = new MapTypeControl();
  //添加到地图中
  map.addControl(myZoomCtrl);
}