import getIcon from "./icon";
import("./marker.css");

let myRichInfo = null;

// 新建一个marker
export function initMarker({
  map,
  BMapGL,
  emitMarkerClick,
  data,
  clickFn,
  infoWidth,
  maxInfoWidth,
  infoImgWidth,
  infoImgHeight,
  infoAnchor,

  imgs,
  markersMap,

  getRichInfo,

  running,
}) {
  if (!data.lng || !data.lat) {
    return null;
  }
  var point = new BMapGL.Point(data.lng, data.lat);
  let icon = getIcon({ imgs, data: data, BMapGL: BMapGL });
  if (running) {
    const { canvas, w, h, R } = getCanvas({
      data,
    });
    const iconSize = new BMapGL.Size(w, h);
    icon = new BMapGL.Icon(canvas.toDataURL(), iconSize);
    icon.setAnchor(new BMapGL.Size(w / 2, h - R));
  }
  var marker = new BMapGL.Marker(point, { icon }); // 创建标注
  markersMap.set(data.id, { marker, show: true }); // 目前每次渲染是直接清理marker渲染新的
  // 添加marker点击事件
  setMarkerClick({
    marker,
    map,
    BMapGL,
    emitMarkerClick,
    data,
    clickFn,
    infoWidth,
    maxInfoWidth,
    infoImgWidth,
    infoImgHeight,
    infoAnchor,

    getRichInfo,

    running,
  });
  return marker;
}

// 给marker添加点击事件：marker对应位置的弹窗，不使用data的位置，因为点可能会移动位置
export function setMarkerClick({
  marker,
  map,
  BMapGL,
  emitMarkerClick,
  data,
  clickFn,
  infoWidth,
  maxInfoWidth,
  infoImgWidth,
  infoImgHeight,
  infoAnchor,

  getRichInfo,

  running,
}) {
  if (data.noClick) {
    return;
  }
  marker.addEventListener("click", (e) => {
    // 使用settime是因为要处理地图的点击事件
    // const thatmarker = marker
    setTimeout(() => {
      if (myRichInfo) {
        myRichInfo.remove();
      }
      emitMarkerClick({ marker: marker, data, e });
      if (running) {
        return;
      }
      if (clickFn) {
        clickFn({ marker: marker, data, e });
      } else {
        setInfos({
          marker,
          map,
          BMapGL,
          data,
          infoWidth,
          maxInfoWidth,
          infoImgWidth,
          infoImgHeight,
          infoAnchor,
          getRichInfo,
        });
      }
    }, 0);
  });
}

// marker点击后的弹窗设置
export function setInfos({
  marker,
  map,
  BMapGL,
  data,
  infoWidth,
  maxInfoWidth,
  infoImgWidth,
  infoImgHeight,
  infoAnchor,
  getRichInfo,
}) {
  const info = getDesc({
    data,
    infoWidth,
    maxInfoWidth,
    infoImgWidth,
    infoImgHeight,
  });
  const position = marker.getPosition();
  // 这里的BMapGLLib this
  const richMarker = new BMapGLLib.RichMarker(
    info,
    new BMapGL.Point(position.lng, position.lat),
    {
      anchor: new BMapGL.Size(0, infoAnchor), // 偏移量暂时不需要根据data设置
      enableDragging: false,
    }
  );

  myRichInfo = richMarker;
  getRichInfo(myRichInfo); // 使用外部函数获取当前richInfo
  map.addOverlay(richMarker);
}

// 获取弹窗的内容
// TODO: 超链接-直接渲染a无法触发点击事件
export function getDesc({
  data,
  infoWidth,
  maxInfoWidth,
  infoImgWidth,
  infoImgHeight,
}) {
  let info = ""; // 整体内容
  const dataImg = data.img;
  let desc = ""; // 文字部分
  const infos = data.infos || [];

  // 目前图片大小统一一个大小
  var img = dataImg
    ? `<img style='width:${infoImgWidth};height:${infoImgHeight};' src='${dataImg}' />`
    : ""; // 图片部分

  const ifString = typeof infos === "string";
  if (ifString) {
    desc = infos;
  } else {
    infos.forEach((ele) => {
      desc += `
        <div style="" class="enn-map-info-item">
          ${ele[0]}: <span style='opacity:0.6'>${ele[1]}</span>
        </div>
      `;
    });
  }
  info = `
    <div style="width:${infoWidth};max-width:${maxInfoWidth};">
      ${img}
      <div style='font-size:14px;margin-top:3px;' class="enn-map-info ${
        ifString ? "enn-map-info-string" : ""
      }">
        ${desc}
      </div>
    </div>
  `;

  if (infos.length === 0 && !dataImg) {
    return `<div style="width:max-content;" class="enn-map-info-item">暂无数据</div>`;
  }

  return info;
}

// 运动的点 icon

export function getCanvas({ data }) {
  // 缓存下
  const avatarSize = 80;
  const w = avatarSize;
  const nameHeight = 30;
  const arrowSize = 5;
  const pointMarginY = 6;
  const R = 16; // 提取
  const r = 10;
  let h = avatarSize + nameHeight + arrowSize + pointMarginY + R * 2;

  // 创建 canvas
  const canvas = document.createElement("canvas");
  // DPI 设置
  const scale = document.body.clientWidth / 2408;
  const dpr = window.devicePixelRatio || 1;
  canvas.width = Math.ceil(w * dpr * scale);
  canvas.height = Math.ceil(h * dpr * scale);
  const ctx = canvas.getContext("2d");
  ctx.scale(dpr, dpr);
  ctx.scale(scale, scale);
  ctx.translate(0.5, 0.5);
  // 绘制头像 //
  const iconInfo = data.img;
  if (iconInfo) {
    const img = new Image();
    img.src = iconInfo;
    console.log(img);
    ctx.drawImage(data.canvasImg, 0, 0, avatarSize, avatarSize);
  } else {
    // h -= avatarSize
  }
  // 绘制姓名框
  ctx.translate(0, avatarSize);
  const color = data.strokeColor || "rgb(0,0,200)";
  // console.log(data, '-----');
  ctx.fillStyle = color;
  ctx.fillRect(0, 0, w, nameHeight);
  // 姓名框小箭头
  ctx.moveTo(w / 2 - arrowSize - 5, nameHeight - 5);
  ctx.lineTo(w / 2 + arrowSize + 5, nameHeight - 5);
  ctx.lineTo(w / 2, nameHeight + arrowSize);
  ctx.fill();
  // 姓名
  ctx.fillStyle = "#fff";
  ctx.font = "normal 12px 'PingFang SC'";
  const textRect = ctx.measureText(data.info || "张工");
  const tw = textRect.width;
  const th = textRect.fontBoundingBoxAscent + textRect.fontBoundingBoxDescent;
  ctx.fillText(
    data.info || "张工",
    (w - tw) / 2,
    (nameHeight - th) / 2 + textRect.fontBoundingBoxAscent
  );

  // 绘制圆点
  ctx.translate(w / 2, nameHeight + arrowSize + pointMarginY);
  ctx.fillStyle = color.replace(")", ", .33)");
  ctx.moveTo(0, R);
  ctx.beginPath();
  ctx.arc(0, R, R, 0, Math.PI * 2);
  ctx.fill();
  ctx.fillStyle = color;
  ctx.beginPath();
  ctx.moveTo(0, R);
  ctx.arc(0, R, r, 0, Math.PI * 2);
  ctx.fill();

  return { canvas, w, h, R };
}

export default ({
  // TODO: 添加默认值
  map,
  BMapGL,
  emitMarkerClick,
  data,
  clickFn,
  infoWidth,
  maxInfoWidth,
  infoImgWidth,
  infoImgHeight,
  infoAnchor,

  imgs,
  markersMap,

  getRichInfo,

  running = false,
}) => {
  const marker = initMarker({
    map,
    BMapGL,
    emitMarkerClick,
    data,
    clickFn,
    infoWidth,
    maxInfoWidth,
    infoImgWidth,
    infoImgHeight,
    infoAnchor,

    imgs,
    markersMap,

    getRichInfo,

    running,
  });
  return {
    marker,
  };
};
